using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Vevo.Shared.Domain.SystemServices
{
    // This class allows reading files using "~/" in the path.

    public interface IFileManager
    {
        void Move( string sourceFilePath, string destFilePath );
        void Copy( string sourceFilePath, string destFilePath );
        void Delete( string filePath );
        bool Exists( string filePath );
        bool ContainsUnicodeBom( string filePath );

        DateTime GetFileModifiedDate( string path );
        long GetFileSize( string path );
        string GetFullPath( string path );
        string ReadToEnd( string filePath );
        void SaveAll( string sourceFilePath, string content );

        Stream CreateStream( string filePath, FileMode mode );
        Stream CreateStream( string filePath, FileMode mode, FileAccess access );
        Stream CreateStream( string filePath, FileMode mode, FileAccess access, FileShare share );
        TextReader CreateTextReader( string filePath );
        TextReader CreateTextReader( string filePath, Encoding encoding );
        TextWriter CreateTextWriter( string filePath, bool append );
        TextWriter CreateTextWriter( string filePath, bool append, Encoding encoding );

        // SearchFiles functions return array of files that match search pattern.
        // The returned paths are relative to virtual directory (e.g. "~/App_Data/file1.txt").
        string[] SearchFiles( string path );
        string[] SearchFiles( string path, string searchPattern );
        string[] SearchFiles( string path, string searchPattern, bool includeSubDirectories );
    }
}
