using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Text;
using System.Web;
using System.Xml.Serialization;
using System.Xml;

namespace Vevo.UrlModuleLib.Configuration
{
    [Serializable]
    [XmlRoot( "UrlConfiguration" )]
    public class UrlConfiguration
    {
        private string _incomingRulesPath;
        private string _outgoingRulesPath;
        private UrlRuleCollection _incomingRules;
        private UrlOutgoingRules _outgoingRules;

        private void SetIncomingRule( XmlNode node )
        {
            this._incomingRules = new UrlRuleCollection();
            foreach (XmlNode childNode in node.ChildNodes)
            {
                UrlRule rule = new UrlRule();
                rule.LookFor = childNode["LookFor"].InnerText.Replace( "{", "<" ).Replace( "}", ">" );
                rule.OutData = childNode["OutData"].InnerText;
                rule.SendTo = childNode["SendTo"].InnerText;
                rule.MobileLookFor = childNode["MobileLookFor"].InnerText.Replace( "{", "<" ).Replace( "}", ">" );
                rule.MobileOutData = childNode["MobileOutData"].InnerText;
                rule.MobileSendTo = childNode["MobileSendTo"].InnerText;
                _incomingRules.Add( rule );
            }
        }

        private void SetOutgoingRule( XmlNode node )
        {
            this._outgoingRules = new UrlOutgoingRules();
            this._outgoingRules.CategoryRule = node["CategoryRule"].InnerText;
            this._outgoingRules.DepartmentRule = node["DepartmentRule"].InnerText;
            this._outgoingRules.ProductRule = node["ProductRule"].InnerText;
            this._outgoingRules.TellFriendRule = node["TellFriendRule"].InnerText;
            this._outgoingRules.CustomerReviewRule = node["CustomerReviewRule"].InnerText;
            this._outgoingRules.ContentRule = node["ContentRule"].InnerText;
            this._outgoingRules.ContentMenuRule = node["ContentMenuRule"].InnerText;
            this._outgoingRules.NewsRule = node[ "NewsRule" ].InnerText;
            this._outgoingRules.BlogRule = node[ "BlogRule" ].InnerText;
            this._outgoingRules.BlogListRule = node[ "BlogListRule" ].InnerText;
            this._outgoingRules.BlogCategoryListRule = node["BlogCategoryListRule"].InnerText;
            this._outgoingRules.PromotionRule = node[ "PromoRule" ].InnerText;
            this._outgoingRules.ManufacturerRule = node["ManufacturerRule"].InnerText;
        }

        private UrlConfiguration Load()
        {
            string storeURL = HttpContext.Current.Request.Url.Host + HttpContext.Current.Request.ApplicationPath;
            
            XmlDocument doc = new XmlDocument();
            doc.Load( HttpContext.Current.Server.MapPath( _incomingRulesPath ) );
            XmlNode root = doc.DocumentElement;
            bool found = false;

            foreach (XmlNode node in root.ChildNodes)
            {
                if (!found && node.NodeType == XmlNodeType.Element)
                {
                    if (node.Attributes["URL"].Value.Equals( storeURL ))
                    {
                        found = true;
                        SetIncomingRule( node );
                        break;
                    }
                }
            }

            if (!found)
            {
                foreach (XmlNode node in root.ChildNodes)
                {
                    if (node.NodeType == XmlNodeType.Element)
                    {
                        if (node.Attributes["URL"].Value.Equals( "Default" ))
                        {
                            SetIncomingRule( node );
                            break;
                        }
                    }
                }
            }

            doc = new XmlDocument();
            doc.Load( HttpContext.Current.Server.MapPath( _outgoingRulesPath ) );
            root = doc.DocumentElement;
            found = false;

            foreach (XmlNode node in root.ChildNodes)
            {
                if (!found && node.NodeType == XmlNodeType.Element)
                {
                    if (node.Attributes["URL"].Value.Equals( storeURL ))
                    {
                        found = true;
                        SetOutgoingRule( node );
                        break;
                    }
                }
            }

            if (!found)
            {
                foreach (XmlNode node in root.ChildNodes)
                {
                    if (node.NodeType == XmlNodeType.Element)
                    {
                        if (node.Attributes["URL"].Value.Equals( "Default" ))
                        {
                            SetOutgoingRule( node );
                            break;
                        }
                    }
                }
            }
            
            return this;
        }

        public UrlConfiguration GetConfig()
        {
            UrlConfiguration config = HttpRuntime.Cache["UrlConfiguration"] as UrlConfiguration;
            if (config == null)
            {
                config = Load();
                HttpContext.Current.Cache.Insert( "UrlConfiguration", config );
            }

            return config;
        }

        public UrlRuleCollection IncomingRules
        {
            get
            {
                return _incomingRules;
            }
            set
            {
                _incomingRules = value;
            }
        }

        public UrlOutgoingRules OutgoingRules
        {
            get
            {
                return _outgoingRules;
            }
            set
            {
                _outgoingRules = value;
            }
        }

        public UrlConfiguration( string incomingRulesPath, string outgoingRulesPath )
        {
            _incomingRulesPath = incomingRulesPath;
            _outgoingRulesPath = outgoingRulesPath;
        }
    }
}
