using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Data;
using Vevo.Base.Data.Caching;
using Vevo.Domain.Contents;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentMenuRepository : IContentMenuRepository
    {
        #region Private

        private const string RepositoryName = "ContentMenuRepository";
        private const string GetOneSignature = "GetOne";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string TableName = "ContentMenu";
        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private ContentMenuRepositoryNoCache _repository = new ContentMenuRepositoryNoCache();

        #endregion

        #region Constructors

        public ContentMenuRepository()
        {
        }

        public ContentMenuRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods
        public ContentMenu Save( ContentMenu contentmenu )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ContentMenu newContentMenu = _repository.Save( contentmenu );
            _dataCache.ClearRepositoryCache();
            return newContentMenu;
        }

        public ContentMenu GetOne( string contentmenuID )
        {
            ContentMenu result = (ContentMenu) _dataCache.GetValue( GetOneSignature, _cacheDependency, contentmenuID );

            if (result == null)
            {
                result = _repository.GetOne( contentmenuID );
                _dataCache.SetValue( GetOneSignature, result, contentmenuID );
            }

            return result;
        }


        public void Delete( string contentmenuID )
        {
            _repository.Delete( contentmenuID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                    GetTableSchemaSignature, _cacheDependency
                    );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        #endregion
    }
}
