using System;
using System.Collections.Generic;
using Vevo.Domain.Base;

namespace Vevo.Domain.Contents
{
    public class News : ILocaleLoader, ICloneable
    {
        #region Private

        private string _newsID = "0";
        private string _imageFile = String.Empty;
        private bool _isHotNews = false;
        private DateTime _newsDate = DateTime.Today;
        private bool _isEnabled = false;

        private Culture _defaultCulture;
        private LocaleCollection<NewsLocale> _locales;

        private string _storeID = "1";

        #endregion


        #region Public Constructors

        public News( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<NewsLocale>( this );
        }

        #endregion


        #region Public Properties

        public string NewsID { get { return _newsID; } set { _newsID = value; } }
        public string ImageFile { get { return _imageFile; } set { _imageFile = value; } }
        public bool IsHotNews { get { return _isHotNews; } set { _isHotNews = value; } }
        public DateTime NewsDate { get { return _newsDate; } set { _newsDate = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<NewsLocale> Locales { get { return _locales; } }

        public string StoreID { get { return _storeID; } set { _storeID = value; } }

        public static News Null
        {
            get { return NullNews.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }


        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Topic; }
        }

        public string Topic
        {
            get { return Locales[DefaultCulture].Topic; }
            set { Locales.GetNoLoad( DefaultCulture ).Topic = value; }
        }

        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }

        public string MetaTitle
        {
            get { return Locales[ DefaultCulture ].MetaTitle; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaTitle = value; }
        }

        public string MetaKeyword
        {
            get { return Locales[ DefaultCulture ].MetaKeyword; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaKeyword = value; }
        }

        public string MetaDescription
        {
            get { return Locales[ DefaultCulture ].MetaDescription; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaDescription = value; }
        }
        public string URLName
        {
            get { return Locales[ DefaultCulture ].URLName; }
            set { Locales.GetNoLoad( DefaultCulture ).URLName = value; }
        }
        // End Locale items
        //------------------------------------------------------------
        #endregion

        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (NewsLocale locale in DataAccessContext.NewsRepository.GetLocales( NewsID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            News newObject = (News) MemberwiseClone();
            newObject._locales = (LocaleCollection<NewsLocale>) Locales.Clone();
            return newObject;
        }

        #endregion


        #region Public Methods


        #endregion
    }
}
