using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Configurations;
using Vevo.Domain.Payments;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Domain.Orders
{
    public class Order : ICloneable
    {
        #region Private

        private string _orderID = "0";
        private string _customerID = "0";
        private DateTime _orderDate = DateTime.Today;
        private string _userName = String.Empty;
        private Address _billing = new Address();
        private string _email = String.Empty;
        private ShippingAddress _shipping = new ShippingAddress();
        private string _customerComments = String.Empty;
        private DateTime _shippedDate;
        private string _shippingMethod = String.Empty;
        private string _paymentMethod = String.Empty;
        private string _gatewayPaymentStatus = String.Empty;
        private string _gatewayOrderID = "0";
        private bool _paymentComplete = false;
        private bool _processed = false;
        private string _status = "New";
        private bool _cancelled = false;
        private string _IPAddress = String.Empty;
        private string _couponID = String.Empty;
        private decimal _subtotal = 0;
        private decimal _tax = 0;
        private decimal _shippingCost = 0;
        private decimal _couponDiscount = 0;
        private string _baseCurrecncyCode = String.Empty;
        private string _userCurrencyCode = String.Empty;
        private double _userConversionRate = 1;
        private string _invoiceNotes = String.Empty;
        private string _giftCertificateCode = String.Empty;
        private decimal _giftCertificate = 0;
        private string _trackingNumber = String.Empty;
        private string _trackingMethod = String.Empty;
        private string _giftRegistryID = "0";
        private bool _showShippingAddress = true;
        private string _shippingID = "0";
        private decimal _handlingFee = 0;
        //private string _creditCardNumber = String.Empty;
        //private string _cvv = String.Empty;
        //private string _expiredYear = String.Empty;
        //private string _expiredMonth = String.Empty;
        //private string _creditCardType = String.Empty;
        private DateTime _qbPostDate = DateTime.Today;
        private string _qbReferenceCode = String.Empty;
        private bool _containsRecurring = false;
        private string _parentID = "0";
        private string _paymentToken = String.Empty;
        private string _avsAddrStatue = String.Empty;
        private string _avsZipStatue = String.Empty;
        private string _cvvStatue = String.Empty;
        private string _storeID = "0";
        private string _qBTxnID = String.Empty;
        private bool _isQBExported = false;
        private string _poNumber = String.Empty;
        private bool _istaxExempt = false;
        private string _taxExemptID = String.Empty;
        private string _taxExemptCountry = String.Empty;
        private string _taxExemptState = String.Empty;
        private int _redeemPoint = 0;
        private decimal _redeemPrice = 0;
        private int _pointEarned = 0;
        private bool _isPointApplied = true;
        private bool _isSubscriptionApplied = false;

        private ShippingMethod CreateShippingMethod( Culture culture )
        {
            ShippingOption shippingOption = DataAccessContext.ShippingOptionRepository.GetOne(
                culture, ShippingID );
            ShippingChoice shippingChoice = new ShippingChoice( ShippingMethod, ShippingCost, HandlingFee );

            return shippingOption.CreateShippingMethod( shippingChoice );
        }

        private PaymentMethod CreatePaymentMethod( Culture culture )
        {
            PaymentOption paymentOption = DataAccessContext.PaymentOptionRepository.GetOne(
                culture, PaymentMethod );
            return paymentOption.CreatePaymentMethod();
        }

        #endregion


        #region Constructors

        public Order()
        {
        }

        #endregion


        #region Public Properties

        public string OrderID { get { return _orderID; } set { _orderID = value; } }
        public string CustomerID { get { return _customerID; } set { _customerID = value; } }
        public DateTime OrderDate { get { return _orderDate; } set { _orderDate = value; } }
        public string UserName { get { return _userName; } set { _userName = value; } }
        public Address Billing { get { return _billing; } set { _billing = value; } }
        public string Email { get { return _email; } set { _email = value; } }
        public ShippingAddress Shipping { get { return _shipping; } set { _shipping = value; } }
        public string CustomerComments { get { return _customerComments; } set { _customerComments = value; } }
        public DateTime ShippedDate { get { return _shippedDate; } set { _shippedDate = value; } }
        public string ShippingMethod { get { return _shippingMethod; } set { _shippingMethod = value; } }
        public string PaymentMethod { get { return _paymentMethod; } set { _paymentMethod = value; } }
        public string GatewayPaymentStatus { get { return _gatewayPaymentStatus; } set { _gatewayPaymentStatus = value; } }
        public string GatewayOrderID { get { return _gatewayOrderID; } set { _gatewayOrderID = value; } }
        public bool PaymentComplete { get { return _paymentComplete; } set { _paymentComplete = value; } }
        public bool Processed { get { return _processed; } set { _processed = value; } }
        public string Status { get { return _status; } set { _status = value; } }
        public bool Cancelled { get { return _cancelled; } set { _cancelled = value; } }
        public string IPAddress { get { return _IPAddress; } set { _IPAddress = value; } }
        public string CouponID { get { return _couponID; } set { _couponID = value; } }
        public decimal Subtotal { get { return _subtotal; } set { _subtotal = value; } }
        public decimal Tax { get { return _tax; } set { _tax = value; } }
        public decimal ShippingCost { get { return _shippingCost; } set { _shippingCost = value; } }
        public decimal CouponDiscount { get { return _couponDiscount; } set { _couponDiscount = value; } }
        public string BaseCurrencyCode { get { return _baseCurrecncyCode; } set { _baseCurrecncyCode = value; } }
        public string UserCurrencyCode { get { return _userCurrencyCode; } set { _userCurrencyCode = value; } }
        public double UserConversionRate { get { return _userConversionRate; } set { _userConversionRate = value; } }
        public string InvoiceNotes { get { return _invoiceNotes; } set { _invoiceNotes = value; } }
        public string GiftCertificateCode { get { return _giftCertificateCode; } set { _giftCertificateCode = value; } }
        public decimal GiftCertificate { get { return _giftCertificate; } set { _giftCertificate = value; } }
        public string TrackingNumber { get { return _trackingNumber; } set { _trackingNumber = value; } }
        public string TrackingMethod { get { return _trackingMethod; } set { _trackingMethod = value; } }
        public string GiftRegistryID { get { return _giftRegistryID; } set { _giftRegistryID = value; } }
        public bool ShowShippingAddress { get { return _showShippingAddress; } set { _showShippingAddress = value; } }
        public string ShippingID { get { return _shippingID; } set { _shippingID = value; } }
        public decimal HandlingFee { get { return _handlingFee; } set { _handlingFee = value; } }
        //public string CreditCardNumber { get { return _creditCardNumber; } set { _creditCardNumber = value; } }
        //public string Cvv { get { return _cvv; } set { _cvv = value; } }
        //public string ExpiredMonth { get { return _expiredMonth; } set { _expiredMonth = value; } }
        //public string ExpiredYear { get { return _expiredYear; } set { _expiredYear = value; } }
        //public string CreditCardType { get { return _creditCardType; } set { _creditCardType = value; } }
        public DateTime QbPostDate { get { return _qbPostDate; } set { _qbPostDate = value; } }
        public string QbReferenceCode { get { return _qbReferenceCode; } set { _qbReferenceCode = value; } }
        public bool ContainsRecurring { get { return _containsRecurring; } set { _containsRecurring = value; } }
        public string ParentID { get { return _parentID; } set { _parentID = value; } }
        public string PaymentToken { get { return _paymentToken; } set { _paymentToken = value; } }
        public string AvsAddrStatus { get { return _avsAddrStatue; } set { _avsAddrStatue = value; } }
        public string AvsZipStatus { get { return _avsZipStatue; } set { _avsZipStatue = value; } }
        public string CvvStatus { get { return _cvvStatue; } set { _cvvStatue = value; } }
        public string StoreID { get { return _storeID; } set { _storeID = value; } }
        public string QBTxnID { get { return _qBTxnID; } set { _qBTxnID = value; } }
        public bool IsQBExported { get { return _isQBExported; } set { _isQBExported = value; } }
        public string PONumber { get { return _poNumber; } set { _poNumber = value; } }
        public bool IsTaxExempt { get { return _istaxExempt; } set { _istaxExempt = value; } }
        public string TaxExemptID { get { return _taxExemptID; } set { _taxExemptID = value; } }
        public string TaxExemptCountry { get { return _taxExemptCountry; } set { _taxExemptCountry = value; } }
        public string TaxExemptState { get { return _taxExemptState; } set { _taxExemptState = value; } }
        public int RedeemPoint { get { return _redeemPoint; } set { _redeemPoint = value; } }
        public decimal RedeemPrice { get { return _redeemPrice; } set { _redeemPrice = value; } }
        public int PointEarned { get { return _pointEarned; } set { _pointEarned = value; } }
        public bool IsPointApplied { get { return _isPointApplied; } set { _isPointApplied = value; } }
        public bool IsSubscriptionApplied { get { return _isSubscriptionApplied; } set { _isSubscriptionApplied = value; } }

        public IList<OrderItem> OrderItems
        {
            get
            {
                return DataAccessContext.OrderItemRepository.GetByOrderID( OrderID );
            }
        }

        public decimal Total
        {
            get
            {
                decimal total = Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - RedeemPrice - GiftCertificate;
                if (total < 0)
                    total = 0;

                return total;
            }
        }

        public bool ContainsProductOnly
        {
            get
            {
                foreach (OrderItem orderItem in OrderItems)
                {
                    if ((orderItem.ProductID == "0") || (String.IsNullOrEmpty( orderItem.ProductID )))
                    {
                        return false;
                    }
                }
                return true;
            }
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            Order newObject = (Order) MemberwiseClone();
            return newObject;
        }

        #endregion


        #region Public Methods

        public void SetOrderAmount( OrderAmount orderAmount )
        {
            Subtotal = orderAmount.Subtotal;
            CouponDiscount = orderAmount.Discount;
            RedeemPrice = orderAmount.PointDiscount;
            ShippingCost = orderAmount.ShippingCost;
            HandlingFee = orderAmount.HandlingFee;
            Tax = orderAmount.Tax;
            GiftCertificate = orderAmount.GiftCertificate;
        }

        public CheckoutDetails CreateCheckoutDetails(
            Culture culture )
        {
            CheckoutDetails checkout = new CheckoutDetails();

            checkout.BillingAddress = Billing;
            checkout.ShippingAddress = Shipping;

            checkout.Email = Email;
            checkout.CustomerComments = CustomerComments;

            checkout.Coupon = DataAccessContext.CouponRepository.GetOne( CouponID );

            checkout.ShippingMethod = CreateShippingMethod( culture );
            checkout.ShowShippingAddress = ShowShippingAddress;

            checkout.GiftCertificate = DataAccessContext.GiftCertificateRepository.GetOne( GiftCertificateCode );

            checkout.PaymentMethod = CreatePaymentMethod( culture );

            checkout.GiftRegistryID = GiftRegistryID;
            checkout.PaymentToken = PaymentToken;

            checkout.IsTaxExempt = IsTaxExempt;
            checkout.TaxExemptID = TaxExemptID;
            checkout.TaxExemptCountry = TaxExemptCountry;
            checkout.TaxExemptState = TaxExemptState;
            checkout.RedeemPoint = RedeemPoint;
            checkout.RedeemPrice = RedeemPrice;

            return checkout;
        }

        public Cart CreateShoppingCart()
        {
            Cart shoppingCart = new Cart();

            foreach (OrderItem orderItem in OrderItems)
            {
                // TO DO: Need to be able to restore option item IDs from Order

                string giftCertificateCode = DataAccessContext.GiftCertificateRepository.GetCodeFromOrderItemID(
                    orderItem.OrderItemID );
                GiftCertificate giftCertificate = DataAccessContext.GiftCertificateRepository.GetOne(
                    giftCertificateCode );

                OrderCartItem orderCartItem = new OrderCartItem(
                    shoppingCart.CartID,
                    DataAccessContext.ProductRepository.GetOne( Culture.Null, orderItem.ProductID, StoreID ),
                    orderItem.Quantity,
                    orderItem.UnitPrice,
                    OptionItemValueCollection.Null,
                    giftCertificate.CreateCartItemGiftDetails() );

                shoppingCart.AddItem( orderCartItem );
            }

            return shoppingCart;
        }

        #endregion
    }
}
