﻿using System;
using System.Collections.Generic;
using Vevo.Domain.Base;

namespace Vevo.Domain.Products
{
    public enum SpecificationItemControlType
    { 
        Textbox,
        DropDownList,
        MultiSelect
    }
    [Serializable]
    public class SpecificationItem : ILocaleLoader, ICloneable
    {
        #region Private

        private string _specificationItemID = "0";
        private string _specificationGroupID = "0";
        private string _name = String.Empty;
        private SpecificationItemControlType _type = SpecificationItemControlType.Textbox;
        private bool _useInFacetedSearch = false;

        private Culture _defaultCulture;
        private LocaleCollection<SpecificationItemLocale> _locales;

        #endregion

        #region Public Constructors

        public SpecificationItem( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<SpecificationItemLocale>( this );
        }

        #endregion

        #region Public Properties

        public static SpecificationItem Null
        {
            get { return NullSpecificationItem.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string SpecificationItemID { get { return _specificationItemID; } set { _specificationItemID = value; } }
        public string SpecificationGroupID { get { return _specificationGroupID; } set { _specificationGroupID = value; } }
        public string Name { get { return _name; } set { _name = value; } }
        public SpecificationItemControlType Type { get { return _type; } set { _type = value; } }
        public bool UseInFacetedSearch { get { return _useInFacetedSearch; } set { _useInFacetedSearch = value; } }
        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<SpecificationItemLocale> Locales { get { return _locales; } }


        //------------------------------------------------------------
        // Locale items
        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }

        public string DisplayName
        {
            get { return Locales[DefaultCulture].DisplayName; }
            set { Locales.GetNoLoad( DefaultCulture ).DisplayName = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion

        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (SpecificationItemLocale locale in DataAccessContext.SpecificationItemRepository.GetLocales( SpecificationItemID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            SpecificationItem newObject = (SpecificationItem) MemberwiseClone();
            newObject._locales = (LocaleCollection<SpecificationItemLocale>) Locales.Clone();

            return newObject;
        }

        #endregion
    }
}
