using System;
using System.Data;
using System.Configuration;
using System.Text;


namespace Vevo.WebUI.International
{
    /// <summary>
    /// Summary description for PageTokenProcessor
    /// </summary>
    public class PageTokenProcessor
    {
        private const string TokenStartMarker = "[";
        private const string TokenEndMarker = "]";

        private const string TokenStartMarkerLong = "%5B";
        private const string TokenEndMarkerLong = "%5D";

        private const char TokenLiteralMarker = '$';


        private LanguageTokenRule _rule;
        private string _source;
        private StringBuilder _output;

        private string _currentStartMarker;
        private string _currentEndMarker;


        public PageTokenProcessor( LanguageTokenRule rule )
        {
            _rule = rule;
        }


        private bool IsTokenStartMarker( int index )
        {
            if ((index + TokenStartMarker.Length < _source.Length && 
                _source.Substring( index, TokenStartMarker.Length ) == TokenStartMarker) ||

                (index + TokenStartMarkerLong.Length < _source.Length &&
                _source.Substring( index, TokenStartMarkerLong.Length ) == TokenStartMarkerLong))
            {
                if (_source.Substring( index, TokenStartMarker.Length ) == TokenStartMarker)
                {
                    _currentStartMarker = TokenStartMarker;
                    _currentEndMarker = TokenEndMarker;
                }
                else
                {
                    _currentStartMarker = TokenStartMarkerLong;
                    _currentEndMarker = TokenEndMarkerLong;
                }

                switch (_source[index + _currentStartMarker.Length])
                {
                    case TokenLiteralMarker:
                        return true;

                    default:
                        return false;
                }
            }
            else
            {
                return false;
            }
        }

        private int GetNonTokenTextLength( int index )
        {
            int endIndex = index;
            while (endIndex < _source.Length &&
                   !IsTokenStartMarker( endIndex ))
            {
                endIndex++;
            }

            return endIndex - index;
        }

        private string ExtractToken( int index, out int length )
        {
            length = 0;

            if (index + _currentStartMarker.Length + _currentEndMarker.Length + 1 > _source.Length)
                return null;

            if ((_source.Substring( index, TokenStartMarker.Length ) == TokenStartMarker &&
                _source[index + 1] == TokenLiteralMarker) ||

                (_source.Substring( index, TokenStartMarkerLong.Length ) == TokenStartMarkerLong &&
                _source[index + TokenStartMarkerLong.Length] == TokenLiteralMarker))
            {
                int endIndex = _source.IndexOf( _currentEndMarker, index + _currentStartMarker.Length + 1 );
                if (endIndex == -1)
                    return null;

                length = endIndex - index + _currentEndMarker.Length;
                return _source.Substring(
                    index + _currentStartMarker.Length + 1,
                    endIndex - index - _currentStartMarker.Length - 1 ).Trim();
            }
            else
            {
                return null;
            }
        }

        private string GetReplacementText( string token )
        {
            return _rule.GetReplacementText( token );
        }


        private int EatNonTokenText( int index )
        {
            int length = GetNonTokenTextLength( index );
            _output.Append( _source, index, length );

            return index + length;
        }

        private int ReplaceToken( int index )
        {
            int length;
            string text = null;

            string token = ExtractToken( index, out length );
            if (token != null)
                text = GetReplacementText( token );

            if (token == null || text == null)
            {
                // Go pass start of token marker
                length = _currentStartMarker.Length + 1;
                _output.Append( _source, index, length );
            }
            else
            {
                _output.Append( text );
            }

            return index + length;
        }


        public StringBuilder Process( string source )
        {
            _source = source;
            _output = new StringBuilder();

            int index = 0;
            while (index < _source.Length)
            {
                index = EatNonTokenText( index );

                if (index < _source.Length)
                    index = ReplaceToken( index );
            }

            return _output;
        }

    }
}
