using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Deluxe.Domain.Orders;

namespace Vevo.WebUI.Orders
{
    public class CartDisplayService
    {
        #region Private

        private string GenerateTrialOrderAmountText(
            Currency currency,
            OrderAmount trialOrderAmount,
            int recurringNumberOfTrialCycles )
        {
            StringBuilder sb = new StringBuilder();

            if (recurringNumberOfTrialCycles > 0)
            {
                if (trialOrderAmount.Discount > 0)
                    sb.AppendFormat( "[$TrialDiscountLabel] {0}\n", currency.FormatPrice( trialOrderAmount.Discount ) );
                if (trialOrderAmount.ShippingCost > 0)
                    sb.AppendFormat( "[$TrialShippingCostLabel] {0}\n", currency.FormatPrice( trialOrderAmount.ShippingCost ) );
                if (trialOrderAmount.HandlingFee > 0)
                    sb.AppendFormat( "[$TrialHandlingFeeLabel] {0}\n", currency.FormatPrice( trialOrderAmount.HandlingFee ) );
                if (trialOrderAmount.Tax > 0)
                    sb.AppendFormat( "[$TrialTaxLabel] {0}\n", currency.FormatPrice( trialOrderAmount.Tax ) );
                sb.AppendLine();
            }

            return sb.ToString();
        }

        private string GenerateRegularOrderAmountText(
            Currency currency,
            OrderAmount regularOrderAmount )
        {
            StringBuilder sb = new StringBuilder();

            if (regularOrderAmount.Discount > 0)
                sb.AppendFormat( "[$RegularDiscountLabel] {0}\n", currency.FormatPrice( regularOrderAmount.Discount ) );
            if (regularOrderAmount.ShippingCost > 0)
                sb.AppendFormat( "[$RegularShippingCostLabel] {0}\n", currency.FormatPrice( regularOrderAmount.ShippingCost ) );
            if (regularOrderAmount.HandlingFee > 0)
                sb.AppendFormat( "[$RegularHandlingFeeLabel] {0}\n", currency.FormatPrice( regularOrderAmount.HandlingFee ) );
            if (regularOrderAmount.Tax > 0)
                sb.AppendFormat( "[$RegularTaxLabel] {0}\n", currency.FormatPrice( regularOrderAmount.Tax ) );

            return sb.ToString();
        }

        private string GenerateRecurringOrderDescriptionString(
            ICartItem item,
            OrderAmount regularOrderAmount,
            OrderAmount trialOrderAmount,
            Currency currency )
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine( "[$RecurringProductDescription]" );
            sb.AppendLine( "[$RecurringOrderPriceDescription]" );
            sb.AppendLine();

            sb.Append( GenerateTrialOrderAmountText(
                currency, trialOrderAmount, item.RecurringNumberOfTrialCycles ) );

            sb.Append( GenerateRegularOrderAmountText(
                currency, regularOrderAmount ) );

            return sb.ToString();
        }

        #endregion


        #region Public Methods

        public string GetCheckoutTooltipMainText()
        {
            return "[$RecurringTooltipMainText]";
        }

        public string GetCheckoutTooltipBody(
            CheckoutDetails checkout,
            CartItemGroup cartItemGroup,
            Customer customer,
            Currency currency )
        {
            OrderCalculator orderCalculator = new OrderCalculator();

            OrderAmount trialOrderAmount = orderCalculator.Calculate(
                checkout,
                cartItemGroup,
                customer,
                0 ).Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                checkout,
                cartItemGroup,
                customer ) );

            OrderAmount regularOrderAmount = orderCalculator.Calculate(
                checkout,
                cartItemGroup.CreateRegularRecurringGroup(),
                customer,
                0 ).Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                checkout,
                cartItemGroup,
                customer ) );

            ICartItem item = cartItemGroup.CartItems[0];

            return GenerateRecurringOrderDescriptionString(
                item,
                regularOrderAmount,
                trialOrderAmount,
                currency );
        }

        #endregion

    }
}
