using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.Deluxe.Domain.BundlePromotion;
using Vevo.Deluxe.Domain.Products;

namespace Vevo.WebUI.Products
{
    /// <summary>
    /// Summary description for BaseProductUserControl
    /// </summary>
    public partial class BaseProductUserControl : Vevo.WebUI.International.BaseLanguageUserControl
    {
        #region Private

        private Control _optionGroupDetailsParent;
        private Control _addToCartNotificationParent;

        private Product GetProduct( Control bindingContainer )
        {
            return (Product) DataBinder.GetDataItem( bindingContainer );
        }

        #endregion


        #region Protected

        protected Control OptionGroupDetailsParent
        {
            get
            {
                if (_optionGroupDetailsParent == null)
                    _optionGroupDetailsParent = this;

                return _optionGroupDetailsParent;
            }
            set
            {
                _optionGroupDetailsParent = value;
            }
        }

        protected Control AddToCartNotificationParent
        {
            get
            {
                if (_addToCartNotificationParent == null)
                    _addToCartNotificationParent = this;

                return _addToCartNotificationParent;
            }
            set
            {
                _addToCartNotificationParent = value;
            }
        }

        protected IOptionGroupDetailsControl uxOptionGroupDetailsProxy
        {
            get
            {
                return (IOptionGroupDetailsControl) OptionGroupDetailsParent.FindControl( 
                    "uxOptionGroupDetails" );
            }
        }

        protected IAddToCartNotificationControl uxAddToCartNotificationProxy
        {
            get
            {
                IAddToCartNotificationControl control =
                    (IAddToCartNotificationControl) AddToCartNotificationParent.FindControl( "uxAddToCartNotification" );

                if (control != null)
                    return control;
                else
                    return (IAddToCartNotificationControl) AddToCartNotificationParent;
            }
        }

        protected void uxTellFriendButton_Command( object sender, CommandEventArgs e )
        {
            string tellFriendUrl = e.CommandArgument.ToString();
            Response.Redirect( tellFriendUrl );
        }

        protected bool IsCallForPrice( object isCallForPrice )
        {
            return ConvertUtilities.ToBoolean( isCallForPrice );
        }

        protected bool IsFixedPrice( object isFixedPrice )
        {
            return IsFixedPrice( isFixedPrice, false );
        }

        protected bool IsFixedPrice( object isFixedPrice, object isCustomPrice )
        {
            return IsFixedPrice( isFixedPrice, isCustomPrice, false );
        }

        protected bool IsFixedPrice( object isFixedPrice, object isCustomPrice, object isCallForPrice )
        {
            if (!IsAuthorizedToViewPrice())
                return false;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isCustomPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isFixedPrice ))
                return true;
            else
                return false;
        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, decimal retailPrice )
        {
            return IsRetailPriceEnabled( isFixedPrice, false, retailPrice );
        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, object isCustomPrice, decimal retailPrice )
        {
            return IsRetailPriceEnabled( isFixedPrice, isCustomPrice, retailPrice, false );

        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, object isCustomPrice, decimal retailPrice, object isCallForPrice )
        {
            if (!IsAuthorizedToViewPrice())
                return false;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isCustomPrice ))
                return false;

            if (CatalogUtilities.IsRetailMode &&
                ConvertUtilities.ToBoolean( isFixedPrice ) &&
                ConvertUtilities.ToDecimal( retailPrice ) > 0
                )
                return true;
            else
                return false;
        }

        protected decimal GetRetailPrice( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, (string) productID, StoreContext.CurrentStore.StoreID );
            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            return productPrice.RetailPrice;
        }

        protected string GetDiscountPercent( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, (string) productID, StoreContext.CurrentStore.StoreID );
            decimal displayPrice = product.GetDisplayedPrice( StoreContext.WholesaleStatus );
            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            if (displayPrice > 0 && productPrice.RetailPrice > 0)
            {
                decimal discount = (decimal) (((productPrice.RetailPrice - displayPrice) / productPrice.RetailPrice) * 100);
                if (discount < 1)
                    return "0%";
                else
                    return ((int) Math.Ceiling( discount )).ToString() + "%";
            }
            else
                return "0%";
        }

        protected bool IsDiscount( object productID )
        {
            if (GetDiscountPercent( productID ) != "0%")
                return true;
            else
                return false;

        }

        protected bool CheckEqualPrice( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne(
                      StoreContext.Culture,
                      productID.ToString(), StoreContext.CurrentStore.StoreID );

            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            if (productPrice.RetailPrice.Equals( productPrice.Price ) || productPrice.RetailPrice == 0)
                return false;
            else
                return true;
        }

        protected string RemainingStock( object stock )
        {
            int amount = Convert.ToInt32( stock ) - DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" );
            if (amount > 0)
            {
                return amount.ToString();
            }
            else
            {
                amount = 0;
                return amount.ToString();
            }
        }

        protected bool ShowRemainingQuantity( object useInventory )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseStockControl" ) &&
                DataAccessContext.Configurations.GetBoolValue( "ShowQuantity" ) &&
                ConvertUtilities.ToBoolean( useInventory ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        protected bool IsTaxInclude()
        {
            return DataAccessContext.Configurations.GetBoolValue( "TaxIncludedInPrice" ) &&
                !String.IsNullOrEmpty( ConvertUtilities.ToString( Eval( "TaxClassID" ) ) ) &&
                ConvertUtilities.ToString( Eval( "TaxClassID" ) ) != "0";
        }

        protected bool IsDynamicProductKitPrice( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            return currentProduct.IsDynamicProductKitPrice;
        }

        protected bool IsVisibilityRecurringPeriod()
        {
            return (ConvertUtilities.ToBoolean( Eval( "IsRecurring" ) )
                && !CatalogUtilities.IsOutOfStock( Eval( "SumStock" ), Eval( "UseInventory" ) )
                && CheckModeAndPermission());
        }

        protected bool CheckModeAndPermission()
        {
            return !CatalogUtilities.IsCatalogMode();
        }

        protected bool IsVisibilityTrialPeriodMore()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) > 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) > 0m);
        }

        protected bool IsVisibilityTrialPeriod()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) == 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) > 0m);
        }

        protected bool IsVisibilityFreeTrialPeriod()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) > 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) == 0m);
        }

        protected bool IsVisibilityFreeTrialPeriodMore()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) == 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) == 0m);
        }

        protected string ShowRecurringCycles()
        {
            if (ConvertUtilities.ToBoolean( Eval( "IsRecurring" ) ))
                return String.Format(
                    "{0} {1}s",
                    ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringInterval" ) ),
                    Eval( "ProductRecurring.RecurringIntervalUnit" ).ToString().ToLower() );
            else
                return String.Empty;
        }

        protected string GetFormattedPriceFromContainer( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );

            decimal price = currentProduct.GetDisplayedPrice( StoreContext.WholesaleStatus );

            return StoreContext.Currency.FormatPrice( price );
        }

        protected bool IsAuthorizedToViewPrice()
        {
            return IsAuthorizedToViewPrice( false );
        }

        protected bool IsAuthorizedToViewPrice( object isCallForPrice )
        {
            bool show = true;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (DataAccessContext.Configurations.GetBoolValue( "PriceRequireLogin" ) 
                && !Page.User.Identity.IsAuthenticated)
            {
                show = false;
            }

            return show;
        }

        protected bool IsRmaEmable()
        {
            if (DataAccessContext.Configurations.GetBoolValue( "EnableRMA" ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        protected string RmaMessage( object returnTime )
        {
            int day = Convert.ToInt32( returnTime );

            if (day == 0)
            {
                return GetLanguageText( "NoReturn" );
            }
            else
            {
                return GetLanguageText( "Return" ) + " " + day + " " + GetLanguageText( "ReturnUnit" );
            }
        }

        protected string ImageAlternateText( Control container, string productImageID )
        {
            Product currentProduct = GetProduct( container );
            if (currentProduct == null) return string.Empty;
            foreach (ProductImage image in currentProduct.ProductImages)
            {
                if (image.ProductImageID == productImageID)
                    return image.Locales[StoreContext.Culture].AltTag;
            }
            return string.Empty;
        }

        protected string ImageTitleText( Control container, string productImageID )
        {
            Product currentProduct = GetProduct( container );
            if (currentProduct == null) return string.Empty;
            foreach (ProductImage image in currentProduct.ProductImages)
            {
                if (image.ProductImageID == productImageID)
                    return image.Locales[StoreContext.Culture].TitleTag;
            }
            return string.Empty;
        }

        protected string ImageAlternateText( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            if (currentProduct == null) return string.Empty;
            return currentProduct.GetPrimaryProductImage().Locales[StoreContext.Culture].AltTag;
        }

        protected string ImageTitleText( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            if (currentProduct == null) return string.Empty;
            return currentProduct.GetPrimaryProductImage().Locales[StoreContext.Culture].TitleTag;
        }


        #endregion


        #region Contructors

        #endregion
    }

}
